package cn.wisenergy.chnmuseum.party.common.dfs;

import cn.wisenergy.chnmuseum.party.common.mvc.InterfaceException;
import cn.wisenergy.chnmuseum.party.common.util.FileTypeUtil;
import cn.wisenergy.chnmuseum.party.common.util.FileUtil;
import com.github.tobato.fastdfs.domain.fdfs.StorePath;
import com.github.tobato.fastdfs.domain.fdfs.ThumbImageConfig;
import com.github.tobato.fastdfs.domain.proto.storage.DownloadByteArray;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@Component
public class FastDFSUtils {

    private static String dfsFileAccessBasePath;
    private static ThumbImageConfig imageConfig;
    private static FastFileStorageClient storageClient;

    @Value("${dfsFileAccessBasePath:#{null}}")
    public void setDfsFileAccessBasePath(String dfsFileAccessBasePath) {
        FastDFSUtils.dfsFileAccessBasePath = dfsFileAccessBasePath;
    }

    @Resource
    private ThumbImageConfig thumbImageConfig;
    @Resource
    private FastFileStorageClient fastFileStorageClient;

    @PostConstruct
    public void init() {
        FastDFSUtils.storageClient = fastFileStorageClient;
        FastDFSUtils.imageConfig = thumbImageConfig;
    }

    /**
     * 字节流方式上传
     */
    public static Map<String, Object> uploadImage(InputStream inputStream, long fileSize, String fileName) {
        boolean isImage = FileTypeUtil.isImageByExtension(fileName);
        if (isImage) {
            String mimeType = FileUtil.getMimeType(fileName);
            final String storePath = FastDFSUtils.uploadFile(inputStream, fileSize, fileName);
            Map<String, Object> uploadResult = new HashMap<>();
            uploadResult.put("url", dfsFileAccessBasePath + "/" + storePath);
            uploadResult.put("status", true);
            uploadResult.put("message", "文件上传成功！");
            uploadResult.put("title", fileName);
            uploadResult.put("mimeType", mimeType);
            uploadResult.put("size", fileSize);
            return uploadResult;
        }
        throw new InterfaceException("400", "文件不是图片类型");
    }

    public static String uploadFile(InputStream inputStream, long size, String fileName) {
        final StorePath storePath = storageClient.uploadFile(inputStream, size, FilenameUtils.getExtension(fileName), null);
        return dfsFileAccessBasePath + "/" + storePath.getFullPath();
    }

    /**
     * 下载文件(字节数组)
     */
    public static byte[] downloadFile(String fileUrl) {
        String group = fileUrl.substring(0, fileUrl.indexOf("/"));
        String path = fileUrl.substring(fileUrl.indexOf("/") + 1);
        return storageClient.downloadFile(group, path, new DownloadByteArray());
    }

    /**
     * 删除文件
     */
    public static byte[] deleteFile(String fileUrl) {
        String group = fileUrl.substring(0, fileUrl.indexOf("/"));
        String path = fileUrl.substring(fileUrl.indexOf("/") + 1);
        return storageClient.downloadFile(group, path, new DownloadByteArray());
    }

    private static int[] getImageInfo(byte[] bytes) {
        try {
            ByteArrayInputStream baos = new ByteArrayInputStream(bytes);
            BufferedImage image = ImageIO.read(baos);
            int width = image.getWidth();
            int height = image.getHeight();
            return new int[]{width, height};
        } catch (Exception e) {
            log.error("FastDFSUtils.getImageInfo() error:", e);
        }
        return new int[]{0, 0};
    }

//    private static class UploadFileSender implements UploadCallback {
//        private final InputStream is;
//
//        public UploadFileSender(InputStream is) {
//            this.is = is;
//        }
//
//        @Override
//        public int send(OutputStream out) throws IOException {
//            byte[] b = new byte[1024];
//            int readBytes;
//            while ((readBytes = is.read(b)) != -1) {
//                out.write(b, 0, readBytes);
//            }
//            return 0;
//        }
//    }

}
