package cn.wisenergy.chnmuseum.party.model;

import cn.wisenergy.chnmuseum.party.common.validator.groups.Add;
import cn.wisenergy.chnmuseum.party.common.validator.groups.Update;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * <p>
 * 学习内容
 * </p>
 *
 * @author Danny Lee
 * @since 2021-03-23
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@Accessors(chain = true)
@EqualsAndHashCode(callSuper = false)
@TableName("learning_content")
@ApiModel(value = "学习内容", description = "学习内容")
public class LearningContent implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("学习内容ID")
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    @NotNull(message = "学习内容IDID不能为空", groups = {Update.class})
    private String id;

    @ApiModelProperty("学习内容名称")
    @TableField("name")
    @NotBlank(message = "学习内容名称不能为空", groups = {Add.class, Update.class})
    private String name;

    @ApiModelProperty("学习内容宣传图")
    @TableField("cover")
    @NotBlank(message = "学习内容宣传图不能为空", groups = {Add.class, Update.class})
    private String cover;

    @ApiModelProperty("适用范围")
    @TableField("applicable_scope")
    @NotBlank(message = "适用范围不能为空", groups = {Add.class, Update.class})
    private String applicableScope;

    @ApiModelProperty("学习项目ID")
    @TableField("learning_project_id")
    @NotBlank(message = "学习项目ID不能为空", groups = {Add.class, Update.class})
    private String learningProjectId;

    @ApiModelProperty("新增审核状态")
    @TableField("audit_status_add")
    @NotBlank(message = "新增审核状态不能为空", groups = {Add.class, Update.class})
    private String auditStatusAdd;

    @ApiModelProperty("修改审核状态")
    @TableField("audit_status_edit")
    private String auditStatusEdit;

    @ApiModelProperty("删除审核状态")
    @TableField("audit_status_delete")
    private String auditStatusDelete;

    @ApiModelProperty("排序顺序")
    @TableField("sortorder")
    @NotNull(message = "排序顺序不能为空", groups = {Add.class, Update.class})
    private Integer sortorder;

    @ApiModelProperty("创建日期")
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    private LocalDateTime createTime;

    @ApiModelProperty("更新日期")
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateTime;

}
