package cn.wisenergy.chnmuseum.party.web.controller;

import cn.wisenergy.chnmuseum.party.common.validator.groups.Add;
import cn.wisenergy.chnmuseum.party.common.vo.GenericPageParam;
import cn.wisenergy.chnmuseum.party.model.Asset;
import cn.wisenergy.chnmuseum.party.model.AssetType;
import cn.wisenergy.chnmuseum.party.service.AssetService;
import cn.wisenergy.chnmuseum.party.service.AssetTypeService;
import cn.wisenergy.chnmuseum.party.service.CopyrightOwnerService;
import cn.wisenergy.chnmuseum.party.web.controller.base.BaseController;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * <pre>
 * 视频分类 前端控制器
 * </pre>
 *
 * @author Danny Lee
 * @since 2021-03-17
 */
@Slf4j
@RestController
@RequestMapping("/assetType")
@Api(tags = {"视频分类接口"})
public class AssetTypeController extends BaseController {

    @Resource
    private AssetTypeService assetTypeService;

    @Resource
    private AssetService assetService;

    @Resource
    private CopyrightOwnerService copyrightOwnerService;

    @PostMapping(value = "/save")
    @RequiresPermissions("asset:type:save")
    @ApiOperation(value = "添加视频分类", notes = "添加视频分类")
    public Map<String, Object> saveAssetType(@Validated(value = {Add.class}) AssetType assetType) {
        // 保存业务节点信息
        boolean result = assetTypeService.save(assetType);
        // 返回操作结果
        if (result) {
            return getSuccessResult();
        } else {
            // 保存失败
            return getFailResult();
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "修改视频分类信息", notes = "修改视频分类信息")
    public Map<String, Object> updateAssetType(@Validated AssetType assetType) {
        boolean flag = assetTypeService.updateById(assetType);
        if (flag) {
            return getSuccessResult();
        }
        return getFailResult();
    }

    @DeleteMapping(value = "/delete/{id}")
    @RequiresPermissions("asset:type:delete")
    @ApiOperation(value = "根据ID删除视频分类", notes = "根据ID删除视频分类")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "id", value = "标识ID", paramType = "path", dataType = "String")
    })
    public Map<String, Object> deleteAssetType(@PathVariable("id") String id) {
        boolean result1 = assetTypeService.removeById(id);
//        Asset entity = new Asset();
//        entity.setAssetTypeId(null);
        LambdaUpdateWrapper<Asset> updateWrapper = Wrappers.<Asset>lambdaUpdate().eq(Asset::getAssetTypeId, id);
        updateWrapper.set(Asset::getAssetTypeId, null);
        boolean result2 = this.assetService.update(updateWrapper);
        if (result1 && result2) {
            return getSuccessResult();
        }
        return getFailResult();
    }

    @GetMapping("/getList")
    @RequiresPermissions("asset:type:list")
    @ApiOperation(value = "获取视频分类全部列表(无分页)", notes = "获取视频分类全部列表(无分页)")
    public Map<String, Object> getAssetTypeList() {
        List<AssetType> assetTypeList = assetTypeService.list();
        return getResult(assetTypeList);
    }

    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "_index", value = "分页起始偏移量", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "_size", value = "返回条数", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "nameOrCode", value = "名称或编码", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startDate", value = "创建时间-开始", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "创建时间-结束", paramType = "query", dataType = "String")
    })
    @PostMapping("/getPageList")
    @RequiresPermissions("asset:type:page")
    @ApiOperation(value = "获取视频分类分页列表", notes = "获取视频分类分页列表")
    public Map<String, Object> getAssetTypePageList(GenericPageParam genericPageParam) {
        LambdaQueryWrapper<AssetType> queryWrapper = new LambdaQueryWrapper<>();
        // 对名称或编码模糊查询
        if (StringUtils.isNotBlank(genericPageParam.getNameOrCode())) {
            queryWrapper.like(AssetType::getName, genericPageParam.getNameOrCode());
        }
        // 根据创建时间区间检索
        if (genericPageParam.getStartDate() != null && genericPageParam.getEndDate() != null) {
            queryWrapper.ge(AssetType::getCreateTime, genericPageParam.getStartDate().atTime(0, 0, 0))
                    .le(AssetType::getCreateTime, genericPageParam.getEndDate().atTime(23, 59, 59));
        }
        // 设置排序规则
        queryWrapper.orderByDesc(AssetType::getCreateTime);
        // 设置查询内容
        queryWrapper.select(
                AssetType::getId,
                AssetType::getName,
                AssetType::getCreateTime,
                AssetType::getUpdateTime);
        Page<AssetType> page = this.assetTypeService.page(getPage(), queryWrapper);
        for (AssetType assetType : page.getRecords()) {


        }
        return getResult(page);
    }

    @ApiOperation(value = "获取视频分类详情", notes = "获取视频分类详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识ID", dataType = "String", paramType = "path")
    })
    @GetMapping("/get/{id}")
    @RequiresPermissions("asset:type:get:id")
    public Map<String, Object> getById(@PathVariable("id") String id) {
        AssetType assetType = assetTypeService.getById(id);
        return getResult(assetType);
    }

}

